#!/usr/bin/env python3

import os
import os.path
import sys
import shutil

import CommonMark

from pygments import highlight
from pygments.util import ClassNotFound
from pygments.lexers import get_lexer_by_name
from pygments.formatters import HtmlFormatter

BASEDIR   = os.path.abspath(os.getcwd())
SRCDIR    = os.path.join(BASEDIR, 'docsrc')
RESDIR    = os.path.join(BASEDIR, 'res')
DOCDIR    = os.path.join(BASEDIR, 'doc')
IMGSRC    = os.path.join(RESDIR, 'img')
STYLESRC  = os.path.join(RESDIR, 'styles')
FONTSRC   = os.path.join(RESDIR, 'fonts')
IMGDEST   = os.path.join(DOCDIR, 'img')
STYLEDEST = os.path.join(DOCDIR, 'styles')
FONTDEST  = os.path.join(DOCDIR, 'fonts')
HEADER    = os.path.join(RESDIR, 'header.html')
FOOTER    = os.path.join(RESDIR, 'footer.html')

def render(markdown_source):
    formatter = HtmlFormatter(linenos=False, cssclass='source')
    root = CommonMark.Parser().parse(markdown_source)
    renderer = CommonMark.HtmlRenderer(options={'preserve_html': True})
    # renderer.escape = lambda s, preserve_entities=None: s
    walker = root.walker()
    nxt = walker.nxt()
    while nxt is not None:
        node = nxt['node']
        if node.t != 'CodeBlock':
            nxt = walker.nxt()
            continue
        if not hasattr(node, 'literal'):
            nxt = walker.nxt()
            continue
        if len(node.literal) <= 0:
            nxt = walker.nxt()
            continue
        try:
            lexer = get_lexer_by_name(node.info)
            node.literal = highlight(node.literal, lexer, formatter)
        except ClassNotFound:
            print('No lexer for language "{}"'.format(node.info))
        nxt = walker.nxt()
    return renderer.render(root)

def main():
    if not shutil.rmtree.avoids_symlink_attacks:
        sys.stderr.write((
            'The rmtree function on this system is vulnerable to symlink '
            'attacks, exiting.'
        ))
        sys.exit(-1)
    if os.path.isdir(DOCDIR):
        shutil.rmtree(DOCDIR)
    os.mkdir(DOCDIR)
    os.mkdir(IMGDEST)
    os.mkdir(STYLEDEST)
    os.mkdir(FONTDEST)
    print('Installing images')
    for image_file in os.listdir(IMGSRC):
        shutil.copy2(os.path.join(IMGSRC, image_file), IMGDEST)
    print('Installing stylesheets')
    for style_file in os.listdir(STYLESRC):
        shutil.copy2(os.path.join(STYLESRC, style_file), STYLEDEST)
    print('Installing fonts')
    for font_file in os.listdir(FONTSRC):
        shutil.copy2(os.path.join(FONTSRC, font_file), FONTDEST)
    with open(HEADER, 'r', encoding='utf-8') as fobj:
        header = fobj.read()
    with open(FOOTER, 'r', encoding='utf-8') as fobj:
        footer = fobj.read()
    for src_file in os.listdir(SRCDIR):
        if not src_file.endswith('.md'):
            continue
        src_file_path = os.path.join(SRCDIR, src_file)
        dst_file_path = os.path.join(
            DOCDIR, '.'.join((src_file[:-3], 'html'))
        )
        with open(src_file_path, 'r', encoding='utf-8') as in_fobj:
            with open(dst_file_path, 'w', encoding='utf-8') as out_fobj:
                source = in_fobj.read()
                out_fobj.write(header)
                print('Rendering {}'.format(src_file_path))
                out_fobj.write(render(source))
                out_fobj.write(footer)

main()

